<?php

namespace App\Yr\Forecast\Tabular\Time;

/**
 * Temperature
 *
 * @author Joachim M. Giæver (joachim[]giaever.org)
 */
class Temperature extends AbstractUnit implements ConvertableInterface {
    const UNIT_CELCIUS      = 'celcius';
    const UNIT_FAHRENHEIT   = 'fahrenheit';
    const UNIT_KELVIN       = 'kelvin';

    /**
     * @param \SimpleXMLElement $xml XML containing the temperature
     */
    public function __construct(?\SimpleXMLElement $xml) {
        parent::__construct(
            (float)$xml['value'],
            (string)$xml['unit']
        );
    }

    /**
     * {@inheritDoc}
     * @todo support conversion from other types, not just celcius
     */
    public function convertTo(string $unit): self {
        switch ($unit) {
        case UNIT_FAHRENHEIT:
            return $this->mul(new CustomUnit(
                9/5, $unit
            ))->add(new CustomUnit(
                32, $unit
            ));
        case UNIT_KELVIN:
            return $this->add(new CustomUnit(
                273.15, $unit
            ));
        }
    }

    /**
     * Return the ⁰X symbol
     */
    public function getDegree(): string {
        return sprintf("⁰%s", strtoupper(substr($this->getUnit(), 0, 1)));
    }
}

